package org.apache.torque.dsfactory;

/*
 * Copyright 2001-2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import javax.sql.ConnectionPoolDataSource;
import javax.sql.DataSource;

import org.apache.commons.configuration.Configuration;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.torque.Torque;
import org.apache.torque.TorqueException;
import org.apache.torque.pool.TorqueClassicDataSource;

/**
 * A factory that uses the "old" Torque Pools for fetching a DataSource.
 *
 * @author <a href="mailto:jmcnally@apache.org">John McNally</a>
 * @author <a href="mailto:hps@intermeta.de">Henning P. Schmiedehausen</a>
 * @version $Id: TorqueDataSourceFactory.java,v 1.13.2.3 2004/05/20 04:35:14 seade Exp $
 * @deprecated as of version 3.1, use SharedPoolDataSourceFactory
 */
public class TorqueDataSourceFactory
    extends AbstractDataSourceFactory
    implements DataSourceFactory
{

    /** The log. */
    private static Log log = LogFactory.getLog(TorqueDataSourceFactory.class);

    /** The wrapped <code>DataSource</code>. */
    private DataSource ds;

    /**
     * @see org.apache.torque.dsfactory.DataSourceFactory#getDataSource
     */
    public DataSource getDataSource()
    {
        return ds;
    }

    /**
     * @see org.apache.torque.dsfactory.DataSourceFactory#initialize
     */
    public void initialize(Configuration configuration) throws TorqueException
    {
        if (configuration == null)
        {
            throw new TorqueException(
                "Torque cannot be initialized without "
                    + "a valid configuration. Please check the log files "
                    + "for further details.");
        }
        ConnectionPoolDataSource cpds = initCPDS(configuration);
        TorqueClassicDataSource tcds = initTorqueClassic(configuration);
        tcds.setConnectionPoolDataSource(cpds);
        ds = tcds;
    }

    /**
     * Initializes the TorqueClassicDataSource.
     *
     * @param configuration where to read the settings from
     * @throws TorqueException if a property set fails
     * @return a configured <code>TorqueClassicDataSource</code>
     */
    protected TorqueClassicDataSource initTorqueClassic(
        Configuration configuration)
        throws TorqueException
    {
        log.debug("Starting initTorqueClassic");
        TorqueClassicDataSource ds = new TorqueClassicDataSource();

        Configuration c = null;

        c = Torque.getConfiguration().subset(DEFAULT_POOL_KEY);
        applyConfiguration(c, ds);

        c = configuration.subset(POOL_KEY);
        applyConfiguration(c, ds);
        return ds;
    }
}
